// F19_V375_P1_SchroersHope_LightCycleBattle.cpp
//
// 375 - Final Project
// A recreation on the classic Tron arcade game with added gameplay complexities
//
// Game designed to be played by two users using the keyboard.
// Controls and instructions are in the readme.txt file
// 
// This program utilizes openGL
// 
// Date: November 12, 2019
// Author: Hope Schroers

// Include appropriate libraries
#include <iostream>
#include <vector>
#include <ctime>
#include <math.h>
#include <string>
#include <sstream>

using namespace std;

#include <GL/glut.h>

// Include necessary user defined classes
#include "graphicsUtils.h"
#include "GamePlay.h"
#include "LightCycle.h"
#include "Trail.h"
#include "Disc.h"
#include "Border.h"

// Define these globally due to glut's implementation

Camera MyCamera;
GamePlay MyGame;
Grid thegrid;

// Declare vector classes with pointers for tanks, barriers and shells
vector< LightCycle* > gCycles;
vector< Trail* > gTrails;
vector< Disc* > gDiscs;
vector< Border* > gBorders;

double elapsedTime; 
clock_t currentTime;
clock_t gPastTime = 0;

int startGame = 1;
// variable used to determine if game with disks is on or off
int discsOn = 0;

// set draw mode to one (true)
int gDrawMode = 1;

void initWindow();
void display();
void reshape( int w, int h );
void moveWorld();
void keyboardDown( unsigned char key, int x, int y );
void keyboardUp( unsigned char key, int x, int y );
// set up a right-click menu
void menu( int value );

int main( int argc, char *argv[] ) 
{
	glutInit(&argc, argv);  
	initWindow();
	MyGame.initWorld();
	
	glutDisplayFunc( display );
	glutReshapeFunc( reshape );
	glutIdleFunc( moveWorld );
	glutIgnoreKeyRepeat( 1 );
    glutKeyboardFunc( keyboardDown );
    glutKeyboardUpFunc( keyboardUp );
	
	// menu with options to clear the game, reset the scores or to exit
	glutCreateMenu( menu );
	glutAddMenuEntry( "Play with Discs", 1 );
	glutAddMenuEntry( "Light Cycles Only", 2 );
	glutAddMenuEntry( "Exit", 3 );
	glutAttachMenu( GLUT_RIGHT_BUTTON );
	
	glutMainLoop();

	return 0;
}

// display()
void display()
{
	glMatrixMode( GL_MODELVIEW );
    MyCamera.doCamera();
        
	// Clear 
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	thegrid.colortable();
	thegrid.drawGrid();
	
	// for each border in the vector of borders, drawBorder
	for ( unsigned int i = 0; i < gBorders.size(); ++i )
	{
			gBorders[i]->drawBorder();
	}
	
	// for each light cycle in the vector of cycles, drawCycle
	for ( unsigned int i = 0; i < gCycles.size(); ++i )
	{
			gCycles[i]->drawCycle();
	}
	
	// for each light trail in the vector of trails, drawTrail
	for ( unsigned int i = 0; i < gTrails.size(); ++i )
	{
			gTrails[i]->drawTrail();
	}
	
	if (discsOn == 0)
	{
		// for each identity disc in the vector of discs, drawDisc
		for ( unsigned int i = 0; i < gDiscs.size(); ++i )
		{
				gDiscs[i]->drawDisc();
		}
	}

	glutSwapBuffers();

}

void moveWorld()
{
	// if user has pressed the start key, begin the game
	if ( startGame == 0 )
	{

        if ( gPastTime == 0 )
        {
                gPastTime = clock();
        }
        else 
        {
                currentTime = clock();
                elapsedTime = double( diffclock( currentTime, gPastTime ) ) / 1000.0;   

                if ( elapsedTime > MINIMUM_MILLISECOND_UPDATE )
                {
                        // for each light cycle in the vector of cycles, compute position and intersection with the other cycle
                        for ( unsigned int i = 0; i < gCycles.size(); ++i )
                        {
                                gCycles[i]->computePosition( elapsedTime );
                                gCycles[i]->computeIntersection();
                                gCycles[i]->endGame();
                        }
                        
                        if (discsOn == 0)
						{
							// for each shell in the vector of shells, compute position and collision with light cycles
							for ( unsigned int i = 0; i < gDiscs.size(); ++i )
							{
								gDiscs[i]->computePosition( elapsedTime, i );
								gDiscs[i]->computeHitCycle(i);
							}
                        }
                        
                        // for each border in the vector of borders, compute intersection with discs and light cycles
						for ( unsigned int i = 0; i < gBorders.size(); ++i )
						{
								gBorders[i]->computeIntersection( discsOn );
						}
                        
                        // for each light trail, compute intersection with discs and light cycles
						for ( unsigned int i = 1; i < gTrails.size(); ++i )
						{
							gTrails[i]->computeIntersection( discsOn, i);
						}

                        gPastTime = currentTime;

                        glutPostRedisplay();
                }
        }
	}
}

void initWindow()
{
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	
	glutInitWindowSize( 1024, 1024 );
	glutInitWindowPosition( 100, 100 );
	
	// This creates the window with a given title
	glutCreateWindow( "Tron Light Cycle Battle" );

	// select clearing background color
	glClearColor( 0.0, 0.0, 0.0, 0.0 );
	
	glEnable( GL_DEPTH_TEST );
}

// reshape
void reshape( int w, int h )
{
        MyCamera.windowWidth = w;
        MyCamera.windowHeight = h;
        glViewport( 0, 0, w, h);
        MyCamera.doCameraOrthographic( w, h );
}

// right-click menu
void menu( int value )
{
	switch ( value )
	{
	// add discs
	case 1:
		gDrawMode = 1;
		discsOn = 0;
		glutPostRedisplay(); // Marks the current window for needing re-display
		break;
	// light cycles only
	case 2:
		gDrawMode = 1;
		discsOn = 1;
		glutPostRedisplay(); // Marks the current window for needing re-display
		break;
	// exit game
	case 3:
		// return 0 to close program
		exit( 0 );
		break;
	}
}

// keyboardDown
void keyboardDown( unsigned char key, int x, int y )
{
		// key to start game
		if ( key == 't' )
        {
			startGame = 0;
		}
			
        // this will be done for all cycles
        for ( unsigned int i = 0; i < gCycles.size(); ++i )
        {
                gCycles[i]->keyDown( key );
        }
        
}



// keyboardUp
void keyboardUp( unsigned char key, int x, int y )
{
        // this will be done for all tanks
        for ( unsigned int i = 0; i < gCycles.size(); ++i )
        {
                gCycles[i]->keyUp( key );
        }
        
}
